@@ -35,6 +35,7 @@ from botocore.signers import add_generate_db_auth_token
 from botocore.exceptions import ParamValidationError
 from botocore.exceptions import AliasConflictParameterError
 from botocore.exceptions import UnsupportedTLSVersionWarning
+from botocore.exceptions import MissingServiceIdError
 from botocore.utils import percent_encode, SAFE_CHARS
 from botocore.utils import switch_host_with_param
 from botocore.utils import hyphenize_service_id
@@ -280,6 +281,8 @@ def register_retries_for_service(service_data, session,
                      "prefix from model for service %s", service_name)
         return
     service_id = service_data.get('metadata', {}).get('serviceId')
+    if service_id is None:
+        raise MissingServiceIdError(service_name=service_name)
     service_event_name = hyphenize_service_id(service_id)
     config = _load_retry_config(loader, endpoint_prefix)
     if not config:
@@ -955,6 +958,9 @@ BUILTIN_HANDLERS = [
     ('before-call.s3.PutBucketVersioning', conditionally_calculate_md5),
     ('before-call.s3.PutBucketWebsite', conditionally_calculate_md5),
     ('before-call.s3.PutObjectAcl', conditionally_calculate_md5),
+    ('before-call.s3.PutObjectLegalHold', calculate_md5),
+    ('before-call.s3.PutObjectRetention', calculate_md5),
+    ('before-call.s3.PutObjectLockConfiguration', calculate_md5),
 
     ('before-parameter-build.s3.CopyObject',
      handle_copy_source_param),
